/*
 * SHA1-based key derivation function (PBKDF2) for IEEE 802.11i
 * Copyright (c) 2003-2005, Jouni Malinen <j@w1.fi>
 *
 * This software may be distributed under the terms of the BSD license.
 * See README for more details.
 */

#include "sha1-pbkdf2.h"
#include "sha1.h"

int pbkdf2_sha1_f(const char *passphrase, size_t passphrase_len, int iterations, unsigned int count, u8 *digest) {
	unsigned char tmp[SHA1_MAC_LEN] = { 0 }, tmp2[SHA1_MAC_LEN] = { 0 };
	int i, j;

	memcpy(digest, tmp, SHA1_MAC_LEN);

	for (i = 1; i < iterations; i++) {
		if (hmac_sha1((u8 *)passphrase, passphrase_len, tmp, SHA1_MAC_LEN, tmp2)) {
			return -1;
		}

		memcpy(tmp, tmp2, SHA1_MAC_LEN);
		for (j = 0; j < SHA1_MAC_LEN; j++) {
			digest[j] ^= tmp2[j];
		}
	}

	return 0;
}

int pbkdf2_sha1(const char *passphrase, size_t passphrase_len, int iterations, u8 *buf, size_t buflen)
{
	unsigned int count = 0;
	unsigned char *pos = buf;
	size_t left = buflen, plen;
	unsigned char digest[SHA1_MAC_LEN];

	while (left > 0) {
		count++;
		if (pbkdf2_sha1_f(passphrase, passphrase_len, iterations, count, digest)) {
			return -1;
		}

		plen = left > SHA1_MAC_LEN ? SHA1_MAC_LEN : left;
		memcpy(pos, digest, plen);
		pos += plen;
		left -= plen;
	}

	return 0;
}
